C C.Capdevila, Cambridge, November 1998
C 'Mechanism Deformation MAP for ODS alloys
C Grain-Twist Project
C
C Variables:
C ----------
C
C Input parameters
C
C MO,CR,TI   Concentration of molybdenum, Chromium and Titanium in wt%.
C PWT	     Concentration of particles in wt%.
C PRAD       Particle radius.
C PARTD      Density of the particle.
C MATRD      Density of the matrix.
C GSIZE      Grain size of the unrecrystallised alloy in m.
C DDENS      Dislocation density.
C AVAC       Area associated with a vacancy.
C DDIF       Constant of difussivity coefficent.
C QES        Self-difussion activation energy for the alloy.
C SRATE      Strain rate.
C BUGV       Burgers vector.
C SMOD       Shear modulus.
C GSREC      Recrystallised grain in m
C 

       PROGRAM ASHBY_MAP
       IMPLICIT NONE
       CHARACTER*30 MECHA,MECHB,MECHC
       CHARACTER*8 MECHD
       DOUBLE PRECISION CTEMP(100),KTEMP(100),TRATK(100),TRAT(100),
     &           DIFFLOW(10000),PLFLOW(10000),ASHCRI(100),
     &		 SMODT(100),ASHTS(100),CRITICAL(100),
     &           GFLOW(10000),VNET(10000),STRESS(10000),
     &		 SISO1(100),TRAKISO1(100),PBS(100),ASHPB(100),
     &		 PWLOW(10000),SISO2(100),TRAKISO2(100),VNET2(10000)		 

       DOUBLE PRECISION PWT,MATRD,PARTD,PRAD,DDIF,SRATE,SROK,PI,GAR,
     &                  GSIZE,SMOD,BUGV,QES,RGAS,OBSTA,PEIRL,PERC,
     &                  DDENS,AVAC,kk,ST,LAMDA,RMAX,RMIN,SOROWAN1,TM,
     &			POISSON,SOROWAN2,SOROWAN,TSTRESS,SMOKGPA,
     &			SMAX,SR,VF,EC,SMIN,GSREC
C
c       PARAMETER(PI=3.1416590D+00)
      INTEGER I,IL,IJI,IJ,DIFWAY,IP,WAY,GWAY,PB,II2,ILL,I2L,I3L,II,IT
	integer iout
C
	PRINT *,'Wait! Processing...'
C
C Units
C
       	OPEN(1,file='D-MAP.in')
       	OPEN(10,file='D-MAP_borders')
       	OPEN(12,FILE='D-MAP_mechanisms')
       	OPEN(8,FILE='D-MAP_isocurves')
c       open(14,file='pruebas')
C
C This reads data from MAP.DAT
C
	READ(1,*) PRAD
	READ(1,*) GSIZE
        READ(1,*) DDENS
	READ(1,*) DDIF
	READ(1,*) QES
	READ(1,*) SMOD
	READ(1,*) BUGV
	READ(1,*) MATRD
	READ(1,*) PARTD
	READ(1,*) PWT
	READ(1,*) SRATE
	READ(1,*) GSREC
	READ(1,*) GAR
	READ(1,*) IT

C
C Headers
C
	WRITE(10,4)
	WRITE(12,441)
	WRITE(8,88)SRATE
C    
C Constants:
C
        PI=3.1416590D+00
	KK=1.381D-23
	RGAS=8.314510D+00
	POISSON=0.33D+00
	TM=1751.0D+00
	SMOKGPA=83.6648D+00
	SR=SRATE
C
C Area associated with a vacancy
C
	AVAC=PI*(BUGV/2.0D+00)**2.0D+00
C
       I=1
       II=1
       DO 1 IJI = 4,100,3
        KTEMP(I) = (TM/100.0D+00)*IJI
        CTEMP(I) = KTEMP(I)-273.15
C
C Temperature dependence of Shear Modulus (MPa)
C
	IF (KTEMP(I).LE.1173) THEN
		SMODT(I)=(-0.00568182D+00*KTEMP(I))+SMOKGPA
	ELSEIF (KTEMP(I).GT.1173) THEN
		SMODT(I)=(-0.036D+00*KTEMP(I))+119.228D+00
	END IF
	SMODT(I)=1.0D+03*SMODT(I)
C
C Threshold stress (LAMDA is the mean planar particle spacing).
C
	RMAX=2.0D+00*PRAD
	RMIN=2.0D+00*BUGV
	VF=PWT*MATRD/PARTD
	LAMDA=1.25D+00*PRAD*SQRT(2.0D+00*PI/(3.0D+00*VF))
	SOROWAN1=(BUGV/LAMDA)*DLOG(RMAX/RMIN)
	SOROWAN2=(0.83D+00*SMODT(I))/(2.0D+00*PI*SQRT(1.0D+00-POISSON))
	SOROWAN=SOROWAN1*SOROWAN2
	TSTRESS=0.33D+00*SOROWAN
C
C Reset Counters
C
	IL=1
	IP=0
	iout=0
C
C Elastic Collpase for BCC
C
	EC=0.1D+00
	SMAX=EC*SMODT(I)
	SMIN=1.0D-03*SMODT(I)
C
	DO 11 ST=TSTRESS,SMAX
	   STRESS(IL)=ST
C
C Diffusional flow
C
           CALL DIFFUSIONAL(KTEMP(I),DIFFLOW(IL),RGAS,ST,SMODT(I),
     &		DIFWAY,GSREC,LAMDA,GAR,IT)
C
C Dislocation glide 
C
	   CALL GLIDE(KTEMP(I),SMODT(I),ST,BUGV,GFLOW(IL),GWAY,
     &              LAMDA,PRAD,SMOKGPA,OBSTA,PEIRL)
	   IF (OBSTA.GE.PEIRL) THEN
		SROK=OBSTA
	   ELSE
		SROK=PEIRL
	   END IF
C
C Power-Law calculation 
C
           CALL POWER(KTEMP(I),PLFLOW(IL),RGAS,SMODT(I),SMOKGPA,ST,
     &		BUGV,WAY,TSTRESS,PWLOW(IL))
	   IL=IL+1
11	CONTINUE
C
	IJ=0
	PB=0
	DO 111 II2=1,(IL-1)
c
c Plastic border
c
	   IF (PLFLOW(1).LT.GFLOW(1)) THEN
		IF ((PLFLOW(II2-1).LT.GFLOW(II2-1)).AND.(PLFLOW(II2+1)
     &			.GT.GFLOW(II2+1))) PB=PB+1
	   END IF	   
	   IF (PLFLOW(1).GT.GFLOW(1)) THEN
		IF ((PLFLOW(II2-1).GT.GFLOW(II2+1)).AND.(PLFLOW(II2+1)
     &			.LT.GFLOW(II2+1))) PB=PB+1
	   END IF
	   IF (PLFLOW(II2).EQ.GFLOW(II2)) PB=PB+1	   
	   IF (PB.EQ.1) PBS(I)=STRESS(II2)
C
C Calculation of Border Field
C
	   IF (PWLOW(1).LT.DIFFLOW(1)) THEN
		IF ((PWLOW(II2-1).LT.DIFFLOW(II2-1)).AND.(PWLOW(II2+1)
     &			.GT.DIFFLOW(II2+1))) IJ=IJ+1
	   END IF	   
	   IF (PWLOW(1).GT.DIFFLOW(1)) THEN
		IF ((PWLOW(II2-1).GT.DIFFLOW(II2-1)).AND.(PWLOW(II2+1)
     &			.LT.DIFFLOW(II2+1))) IJ=IJ+1
	   END IF
	   IF (PWLOW(II2).EQ.DIFFLOW(II2)) IJ=IJ+1	   
	   IF (IJ.EQ.1) CRITICAL(I)=STRESS(II2)
111	CONTINUE
C
	I2L=1
	I3L=1
	DO 1111 ILL=2,(IL-2)
C
C Calculation iso-strain rates curves under plastic field
C
	   IF ((STRESS(ILL).GE.STRESS(1)).AND.
     &                             (STRESS(ILL).LT.PBS(I))) THEN
		VNET2(I2L)=PWLOW(ILL)+DIFFLOW(ILL)
	        IF ((VNET2(I2L).GT.(SRATE*2.5D-01)).AND.(VNET2(I2L)
     &                             .LT.(SRATE*1.25D+00))) THEN
		     SISO2(I)=STRESS(ILL)/(SQRT(3.0D+00)*SMODT(I))
		     TRAKISO2(I)=KTEMP(I)/TM
	        END IF
		I2L=I2L+1
	   ENDIF
C
C Calculation iso-strain rates curves in plastic field
C	 
	   IF ((STRESS(ILL).GE.PBS(I)).AND.
     &                             (STRESS(ILL).LT.STRESS(IL-2))) THEN
		IF (GFLOW(ILL).GT.PLFLOW(ILL)) THEN
		 	VNET(I3L)=GFLOW(ILL)
	        ELSE
	   	 	VNET(I3L)=PLFLOW(ILL)
	        END IF 
	        IF ((VNET(I3L).GT.(SRATE*2.5D-01)).AND.(VNET(I3L).LT.
     &                     (SRATE*1.25D+00))) THEN
		    SISO1(I)=STRESS(ILL)/(SQRT(3.0D+00)*SMODT(I))
		    TRAKISO1(I)=KTEMP(I)/TM
	        END IF
		I3L=I3L+1
	   ENDIF
1111	CONTINUE
C
C Checking system
C
c	write(14,4444)
c	WRITE(14,*)(ktemp(i)/tm)
c	write(14,44)(stress(1)/smodt(i)),plflow(1),pwlow(1),difflow(1),
c     &         gflow(1),vnet(1),vnet2(1)
c	write(14,44)(stress(il-1)/smodt(i)),plflow(il-1),pwlow(il-1),
c     &		difflow(il-1),gflow(il-1),vnet(i3L-1),vnet2(i2l-1)
c
C
C ----------------------       RESULTS      -------------------------------
C
C Ashby Threshold Stress
C
	ASHTS(I)=TSTRESS/(SQRT(3.0D+00)*SMODT(I))
C
C Ashby field boundaries
C
	ASHCRI(I)=CRITICAL(I)/(SQRT(3.0D+00)*SMODT(I))
	if (ashcri(i).le.ashts(i)) iout=1
	if (iout.eq.1) ashcri(i)=ashts(i)
	ASHPB(I)=PBS(I)/(SQRT(3.0D+00)*SMODT(I))
C
C Iso-strain Curves
C
	SROK=SROK/(SQRT(3.0D+00)*SMODT(1)*1.0D+06)
	IF (I.EQ.1) WRITE(8,888)SROK,0.0d+00
	IF (SISO2(I).EQ.0.0D+00) GOTO12
	WRITE(8,888)SISO2(I),TRAKISO2(I)
C
C Melting temperature Tm=1.2Tr, where Tr is recrystallisation T(=1330 C)
C
12	TRAT(I)=CTEMP(I)/1478D+00
	TRATK(I)=KTEMP(I)/TM
C
C Mechanism of dislocation motion
C
	IF (Ij.eq.0) THEN
		MECHA='Diffusional Flow'
		IF (DIFWAY.EQ.1) THEN
			MECHB='Lattice'
		ELSEIF (DIFWAY.EQ.0) THEN
			MECHB='Boundary'
		END IF
	ELSEIF (IJ.NE.0) THEN
		MECHA='Power-Law'
		IF (WAY.EQ.1) THEN
			MECHB='Lattice'
		ELSEIF (WAY.EQ.0) THEN
			MECHB='Core'
		END IF
	END IF
C
C Mechanism of Glide motion
C
	IF (GWAY.EQ.1) THEN
		MECHC='Peierls force'
	ELSEIF (GWAY.EQ.0) THEN
		MECHC='Discrete Obstacles'
	END IF
C
C Mechanism of Diffusional Flow
C
	IF (DIFWAY.EQ.1) THEN
		MECHD='Lattice'
	ELSEIF (DIFWAY.EQ.0) THEN
		MECHD='Boundary'
	END IF
C
C Results are in MPa
C
	WRITE(12,444)TRATK(I),MECHA,MECHB,MECHC,MECHD
	WRITE(10,44)TRATK(I),ASHTS(I),ASHCRI(I),ASHPB(I)
        I = I + 1
C
C Monitoring output data
C
	PERC=(I/35D+00)*100.0D+00
	PERC=INT(PERC)
	IF (PERC.GE.(25*II)) THEN
		WRITE(*,2)(25*II)
		II=II+1
	END IF
	IF (PERC.GE.95) WRITE(*,22)
1      CONTINUE
C
C ----------------------   FORMAT STATEMENTS  -------------------------------
C
2      FORMAT(10x,I4,'%')
22     FORMAT(10x,'Here we go!')
4      FORMAT(4X,'T/Tm',12X,'THRESHOLD',9X,'BORDER-FIELD',5X,
     &       'PLASTICITY-FIELD',/)
112    FORMAT(5(e14.6,5x))
44     FORMAT(10(e14.7,5x))
442    FORMAT(5X,'TEMP',12X,'PLAW',15X,'PLAW2',15X'DIFF',16X,'GLIDE',17X,
     &     'NET',15X,'NET2')
444    FORMAT(e14.7,2x,A16,4x,A10,3X,A16,3X,A8)
441    FORMAT(5X,'T/Tm',6x,'Creep Mecahnism',3x,
     &    'Sub-mecahnism',2x,'Glide Mechanism',4X,'Diffusional')
4444	format(/,'**********')
10     FORMAT(16(E14.7),I1)
88     FORMAT(1X,'Strain Rate=',E14.7,/,4X,'Normalized Stress',4x,
     &    'Homologous Temperature')
888    FORMAT(2(6X,E14.7))
C
       STOP
       END

C
C****************************************************************************
C
C Subroutine to calculate the Strain Rate (1/s) of diffusional flow 
C C.Capdevila, Cambridge, November 1998
C
       SUBROUTINE DIFFUSIONAL(Temp,DF,R,S,SMO,IS,GS,L,GAR,IT)
       IMPLICIT NONE
       INTEGER IS,IT
       DOUBLE PRECISION TEMP,DF,DV0,QV,QB,OMEGA,SS,SMO,SMOK,BB,L,
     &          DB0,DEFF,DV,DB,KK,R,PI,S,T,COBLE,GS,ST1,ST2,RHOB,M,
     &          CONS,GAR         
C
C Constants:
C
	IS=0
	PI=3.1416D+00
	DV0=5.0D-05
	QV=251.0D+03
	QB=175.0D+03
	OMEGA=1.18D-29
	DB0=1.1D-12
	KK=1.381D-23 
	T=TEMP
	SMOK=SMO*1.0D+06
	BB=1.0D-10
C
C Threshold stress grain boundary dislocation migration
C
	ST1=BB*SMO/GS
C
C Threshold stress fine dispersion particles
C
	ST2=1.0D-01*BB*SMO/(3.0D+00*L)
C
C Safety output
C
	IF ((ST1+ST2).GT.S) S=ST1+ST2
C
	SS=(S-ST1-ST2)/SMO
	DB=DB0*DEXP(-QB/(R*T))
	DV=DV0*DEXP(-QV/(R*T))
	IF (IT.EQ.1) GOTO 1
	IF (IT.EQ.2) GOTO 2
	IF (IT.EQ.3) GOTO 3
C
C Equiaxic grain (h = w)
C
3	DEFF=DV+(PI*DB/GS)
	DF=42.0D+00*OMEGA*SMOK*DEFF*SS/(T*KK*(GS*GS))
	GOTO 4
C
C Elongated grain (h ~ w)
C
2	DEFF=(DV/(GAR**2.0D+00))+(PI*DB/(GS*GAR**1.5D+00))
	DF=42.0D+00*OMEGA*SMOK*DEFF*SS/(T*KK*(GS*GS))
	GOTO 4
C
C Elongated grain (h >> w)
C
1	DEFF=(DV/GAR)+(PI*DB/(GS*GAR**0.5D+00))
	DF=16.0D+00*OMEGA*SMOK*DEFF*SS/(T*KK*(GS*GS))
C
C Segregation on grain-boundaries
C
4	RHOB=0.9D+00*(SS/BB)
	M=RHOB*DB*(BB**2.0D+00)*(OMEGA**(0.33d+00))/(KK*T)
	CONS=(21.0D+00*DEFF*OMEGA)/(KK*T*GS*M)
	DF=DF/(1+CONS)
C
	COBLE=PI*DB/GS
	IF (DV.GT.COBLE) IS=1
	RETURN
	END
C
C****************************************************************************
C
C Subroutine to calculate the Strain Rate (1/s) of diffusional flow 
C C.Capdevila, Cambridge, November 1998
C
       SUBROUTINE POWER(TE,PWLAW,R,SMO,SOK,STRESS,BURGV,I,SMIN,PWG)
       IMPLICIT NONE
       INTEGER I
       DOUBLE PRECISION T,POWF,DV0,QV,QC,N,A2,BURGV,TE,SINH,AX,TM,
     &          DC0,DEFF,DV,DC,KK,R,PI,STRESS,SMO,SS,CORE,ALPHA,
     &		SMOK,PWLAW,SOK,SMOT,ST,SMIN,SSTH,PWG,BX,SINHB


          
	T=TE
C
C Constants:
C
      PI=3.1416590D+00
	I=0
	TM=1751.0D+00
C
C Creep exponent according literature results
C
        if ((T/TM).le.0.6) n=7.7d+00
	if (((t/tm).gt.0.6).and.((T/TM).LE.0.8)) then
		N=-18.8D+00*(T/TM)+18.5
	end if
	if ((T/TM).GT.0.8) N=4.0D+00
C
	A2=1.0D+02*(SQRT(3.0D+00)**(N+1.0D+00))
	DV0=5.0D-05
	QV=251.0D+03
	QC=174.0D+03
	DC0=1.0D-23
	KK=1.381D-23
	SMOK=SOK*1.0D+09
	SMOT=SMO*1.0D+06
	ST=SMIN
C
C
	DV=DV0*DEXP(-QV/(R*T))
	DC=DC0*DEXP(-QC/(R*T))
	SS=(STRESS/SMO)
	DEFF=DV+(1.0D+01*DC*SS*SS/(BURGV*BURGV))
C
C Power-law breakdown correction introduced
C
	POWF=(A2*DEFF*SMOT*BURGV)/(KK*T)
	ALPHA=0.7D+03*SMOK/SMOT
	AX=ALPHA*SS
	SINH=(DEXP(AX)-DEXP(-AX))/2.0D+00
	PWLAW=POWF*(SINH/ALPHA)**N
C
C Low stress power-law flow
C
	SSTH=(STRESS-ST)/SMO
	BX=ALPHA*SSTH
	SINHB=(DEXP(BX)-DEXP(-BX))/2.0D+00
	PWG=POWF*(SINHB/ALPHA)**N
C
C Safety output
C
	IF (PWLAW.GE.1.0D+308) PWLAW=1.0D+308
	IF (PWG.GE.1.0D+308) PWG=1.0D+308
C
	CORE=(1.0D-02*DC*SS*SS/(BURGV*BURGV))
	IF (DV.GT.CORE) I=1
1	RETURN
	END
C
C****************************************************************************
C
C Subroutine to calculate the Strain Rate (1/s) dislocation glide 
C C.Capdevila, Cambridge, December 1998
C
       SUBROUTINE GLIDE(TEMP,SM,STRESS,BURGV,GF,I,L,R,SMOKG,GL1,GL2)
       IMPLICIT NONE
       INTEGER I
       DOUBLE PRECISION T,TEMP,SS,SMO,GP,EXP2,SM,L,R,EXP1,GL1,GL2,
     &          G1,KK,STRESS,BURGV ,GF,STRAIN2,THAO1,
     &          ENERGY2,ENERGY1,STRAIN1,THAO2,SMOK,SMOKG
C
C Constants:
C
	I=0
	G1=1.0D+06
	GP=1.0D+14
	KK=1.381D-23
	SMOK=SMOKG*1.0D+09
C
C
	T=TEMP
	SMO=SM
	SS=STRESS
C 
C Strain dued to discrete obstacles
C
	ENERGY1=1.0D-03*SMOK*BURGV*BURGV*R
	THAO1=3.0D+00*(SMOK*BURGV/L)
	EXP1=(1.0D+00-(SS*1.0D+06/THAO1))
	IF ((SS*1.0D+06).GT.THAO1) THEN 
			STRAIN1=G1
			GOTO 11
	ENDIF
	STRAIN1=G1*DEXP(-ENERGY1*EXP1/(KK*T))
C
C Strain dued to lattice resistance
C
11	ENERGY2=0.1D+00*SMOK*(BURGV**3.0D+00)
	THAO2=6.0D-02*SMOK
	IF ((SS*1.0D+06).GT.THAO2) THEN
			STRAIN2=GP*((SS/SMO)**2.0D+00)
			GOTO 22
	ENDIF
	EXP2=1.0D+00-((SS*1.0D+06/THAO2)**(0.75D+00))
	EXP2=EXP2**(1.334D+00)
	STRAIN2=GP*((SS/SMO)**2.0D+00)*DEXP(-ENERGY2*EXP2/(KK*T))
C
C The plasticity strain-rate is the least of Glide Controlled or Pierls
C force mechanism
C
	
22	IF (STRAIN2.LE.STRAIN1) THEN
		GF=STRAIN2
	ELSE
		GF=STRAIN1
	ENDIF
C
	IF (STRAIN2.LT.STRAIN1) I=1
C
	GL1=THAO1
	GL2=THAO2
	RETURN
	END
C
C
